package es.rediris.forja.jleon.arrivalert;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.os.IBinder;
import android.widget.Toast;
import java.util.ArrayList;

import com.google.android.maps.GeoPoint;

public class ArrivAlertService extends Service implements ILocationChangedHandler {
	 
	private LocationListenerArrivAlert mLocationListener;
	private GeoPoint p1; // Localizacin de referencia 
	private Evento ev;
	public static final int ID_NOTIFICACION = 1;
	private ArrayList<Evento> lista_eventos;

	
	
	@Override
	public void onCreate() {
		Toast.makeText(this,"Servicio creado", Toast.LENGTH_LONG).show();
		this.mLocationListener = new LocationListenerArrivAlert(this);		
		this.lista_eventos = new ArrayList<Evento>();
		this.ev = null;
	} 

	@Override
	public int onStartCommand(Intent i, int flags, int idArranque) {
		Toast.makeText(this,"Servicio iniciado "+ idArranque, Toast.LENGTH_LONG).show();
		
		this.mLocationListener.addLocationChangedListener(this);
		this.mLocationListener.activar();
		
		//Recurrir a BBDD del evento
		Evento c = BDEvento.getCita(this, i.getExtras().getInt(Constantes.CTE_EVENTO_ID));
		
		if(this.ev == null) {
			this.ev = c;			
		}
		else {
			this.lista_eventos.add(c);
		}
		
		// Activar LocationListener
		return START_STICKY;
	}	 
	
	@Override
	public void onDestroy() {
		Toast.makeText(this,"Servicio detenido", Toast.LENGTH_LONG).show();
	}	 
	
	@Override
	public IBinder onBind(Intent intencion) {
		return null;
	}
	

	@Override
	public void LocationChanged(LocationListenerArrivAlert source, GeoPoint lastLocation) {
		// Comprobamos posibles errores
		if(this.ev == null) {
			continuamos();
			return;
		}
		else if(this.ev.getUbicacion() == null) {
			this.ev.setEstado(Evento.ESTADO_PUNTUAL);//TODO
			finalMethod();
			return;
		}
		
		if(this.p1 == null) {
			this.p1 = lastLocation;
			Toast.makeText(this, "ArrivAlert informa que el evento " + this.ev.getNombre() + " est prximo." + "\n" + "Calculando distancia hasta la ubicacin del evento...", Toast.LENGTH_LONG).show();
			return;
		}
		
		// Sino, comprobamos que no estemos muy lejos
		if(Utilidades.getDistanciaKm(this.p1, lastLocation) > 1) {
			this.p1 = lastLocation;
			Toast.makeText(this, "ArrivAlert est comprobando si la distancia hasta el evento " + this.ev.getNombre() + " es correcta...", Toast.LENGTH_LONG).show();
			return;
		}
		

		// Determinar puntualidad
		double dist = Utilidades.getDistanciaKm(lastLocation, this.ev.getUbicacion().getLocation());
		//TODO: Estimacion temporal
		Toast.makeText(this, "ArrivAlert informa que se encuentra a " + dist + " Km. del evento " +  this.ev.getNombre(), Toast.LENGTH_LONG).show();
		
		if(this.ev.llegoTarde(lastLocation))
			this.ev.setEstado(Evento.ESTADO_IMPUNTUAL);//TODO
		else
			this.ev.setEstado(Evento.ESTADO_PUNTUAL);//TODO		

		finalMethod();
	}
	
	
	// Private methods
	
	private void finalMethod() {
		 // Actualizamos el estado de la cita en la bbdd
		BDEvento.updateCita(this, ev);
		
		// Lanzamos la notificacin
		lanzarNotificacion();
		
		// Programamos el BroadcastReceiver que modificar 
		// el estado a finalizado cuando pase la fecha final
		ProgramarAlarmaFinal();
		
		// Detenemos el servicio, ya que ya hemos terminado para esta cita
		continuamos();
	}
	
	
	
	
	private void lanzarNotificacion() {   //TODO
		int idIcon = this.ev.getEstado() == Evento.ESTADO_IMPUNTUAL ?
					 R.drawable.alerta_1 : R.drawable.caledarojo;
		
		 // Creamos Notificacin
		 //String tickerText = ev.getNombre() + ": " + ev.getStrEstado();
		//String tickerText = ev.getNombre();
		 String tickerText = "ArrivAlert informa sobre " + ev.getNombre();
		 NotificationManager nm = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
		 Notification n = new Notification(idIcon, tickerText, System.currentTimeMillis());
		 n.defaults |= Notification.DEFAULT_SOUND;
		 n.vibrate = new long[] {100,250,100,250,100,450};
		 //n.defaults |= Notification.DEFAULT_VIBRATE;
		 n.defaults |= Notification.DEFAULT_LIGHTS;
		 n.flags    |= Notification.FLAG_AUTO_CANCEL;
		 
		 
		 Intent i = new Intent(this, CalendarioEventosActivity.class);
		 //i.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
		 i.putExtra(Constantes.CTE_EVENTO_ID, ev.getId());
		 		 
		 PendingIntent pi = PendingIntent.getActivity(this, 7777, i, 0);		
		 //n.setLatestEventInfo(this, "ArrivAlert informa: " + ev.getStrEstado() , ev.toString(), pi);
		 //n.setLatestEventInfo(this, "ArrivAlert informa: " + ev.getStrEstado() , ev.getNombre(), pi);
		 //n.setLatestEventInfo(this, "ArrivAlert informa sobre " + ev.getNombre() + " : ", ev.getStrEstado(), pi);
		 n.setLatestEventInfo(this, "Evento " + ev.getNombre(), ev.getStrEstado() + " de " + ev.getUbicacion().getNombre(), pi);
		 nm.notify(ev.getId(), n);
		 
		 // Ejecutamos las acciones si llegamos tarde
		 if(ev.getEstado() == Evento.ESTADO_IMPUNTUAL) { //TODO
			 
			 for(Alerta a : ev.getAcciones())
				 a.operar(this, Preferencias.getEnable(this));//TODO
		 }		 
		 
	}
	
	private void ProgramarAlarmaFinal() {
		ArrivAlertReceiver.ReProgramarAlertas(this, this.ev);
	}
	
	private void continuamos() {
		if(this.lista_eventos.size() > 0) {
			this.ev = this.lista_eventos.get(0);
			this.lista_eventos.remove(0);
		}
		else {
			this.ev = null;
			this.p1 = null;
			this.mLocationListener.removeLocationChangedListener(this);
			this.mLocationListener.desactivar();
			this.stopSelf();  
		}
	}
}
