package es.rediris.forja.jleon.arrivalert;

import java.util.ArrayList;

import android.app.ListActivity;
import android.content.Intent;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.Toast;

public class CalendarioEventosActivity extends ListActivity 
{
	// Attributes
	private AdaptarElementos mAdaptador;

	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setContentView(R.layout.calendarioeventos);
		
		//refreshAdaptador();
		
		// Registramos el context menu
	    ListView list = (ListView)this.findViewById(android.R.id.list);
	    registerForContextMenu(list);
	}
	
	@Override
	protected void onResume()
	{
		super.onResume();
		refreshAdaptador();
	}
	
	private void refreshAdaptador()
	{
		// Base de datos actual
		BDEvento bbdd = new BDEvento(this);
        
        // Adaptador 
        ArrayList<IListaElementos> list = new ArrayList<IListaElementos>(bbdd.getAllCitas()); // tenemos que convertir el tipo de lista
        this.mAdaptador = new AdaptarElementos(this, list);
        
        // Set the adapter
        setListAdapter(this.mAdaptador);
        
        bbdd.close();
	}
	
	
	public void onClickOnAdd(View v)
	{		
		// Creamos una nueva Evento
		BDEvento bbdd = new BDEvento(this);
		Evento c = bbdd.insertNewCita(); // la Evento que editaremos en el editor
		bbdd.close();
		
		if(c == null)
		{
			Toast.makeText(this, "Error creating new Appointment", Toast.LENGTH_LONG).show();
			return;
		}
		
		refreshAdaptador(); // actualizamos antes de abrir la nueva actividad

        Intent i = new Intent(this, EventoActivity.class);
        i.putExtra(Constantes.CTE_EVENTO_ID, c.getId());   // pasamos el id de la Evento a actualizar
        this.startActivityForResult(i, Constantes.CTE_REG_EVENTO);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		if(resultCode == RESULT_OK && requestCode == Constantes.CTE_REG_EVENTO)
		{
			refreshAdaptador(); // si ha anyadido bien la nueva Acción, refrescamos desde la base de datos
		}
	}
	
	
	@Override 
	protected void onListItemClick(ListView listView, View view, int position, long id) 
	{
		super.onListItemClick(listView, view, position, id);
		
		Object o = getListAdapter().getItem(position); 							// Cogemos el item en la posición indicada
		
		// Devolvemos el id y el nombre del lugar seleccionado		
		Evento c = (Evento)o;
		editarCita(c);
	}
	
	
	private void seleccionarCita(Evento c)
	{
		Intent i = new Intent();
		i.putExtra(Constantes.CTE_EVENTO_NOMBRE, c.getNombre());
		i.putExtra(Constantes.CTE_EVENTO_ID,     c.getId());
		this.setResult(RESULT_OK, i);
		this.finish();
	}
	
	private void editarCita(Evento c)
	{
		Intent i = new Intent(this, EventoActivity.class);
		i.putExtra(Constantes.CTE_EVENTO_ID, c.getId());
		this.startActivityForResult(i, Constantes.CTE_REG_EVENTO);
	}
	
	private void borrarCita(Evento c)
	{
		// Borramos la fila de la base de datos
		BDEvento bbdd = new BDEvento(this);
        bbdd.deleteCita(c);	        
        bbdd.close();
        
        // refrescamos nuestra lista
        refreshAdaptador();
	}
	
	
	
	// Context Menu
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) 
	{
		if (v.getId()==android.R.id.list) 
		{
			AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)menuInfo;
			Evento c = (Evento)this.mAdaptador.getItem(info.position);
			menu.setHeaderTitle(c.getNombre());
			
			// Add the options
			menu.add(Menu.NONE, 0, 0, R.string.str_select);
			menu.add(Menu.NONE, 1, 1, R.string.str_edit);
			menu.add(Menu.NONE, 2, 2, R.string.str_delete);
		}
	}
	
	
	@Override
	public boolean onContextItemSelected(MenuItem item)
	{
	  AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)item.getMenuInfo();
	  int menuItemId = item.getItemId();
	  Evento c = (Evento)this.mAdaptador.getItem(info.position);
	  
	  switch(menuItemId)
	  {
			case 0: seleccionarCita(c); break;
			case 1: editarCita(c);      break;
			case 2: borrarCita(c);      break;
	  }
	  
	  return true;
	}
	
}
