package es.rediris.forja.jleon.arrivalert;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

import android.text.format.DateFormat;

import com.google.android.maps.GeoPoint;



public class Evento implements Comparable<Evento>, IListaElementos {
	
	private int id;
	private String nombre;
	private String descripcion;
	private Date fecha;
	private Date fechaAlerta;
	private float kmsTarde;
	private int estado;
	private Ubicacion ubicacion;
	private ArrayList<Alerta> alertas;
	
	public static final int ESTADO_FINALIZADO = 1;
	public static final int ESTADO_PENDIENTE = 2;
	public static final int ESTADO_PUNTUAL = 3;
	public static final int ESTADO_IMPUNTUAL = 4;
	
	
	
	public Calendar calendar = Calendar.getInstance(); 
	
	
	/** Variables usadas para crear un contexto **/
	
	public static String STR_ESTADO_TRATADA      = "";
	public static String STR_ESTADO_NO_TRATADA   = "";
	public static String STR_ESTADO_LLEGAS_TARDE = "";
	public static String STR_ESTADO_LLEGAS_EN_HORA = "";
	
	
	// Constructores
	
	public Evento(int 					 id,
				String              	 nombre,
				String                   descripcion,
				Date                     fecha,
				Date                     fechaAlerta,
				float                    kmsTarde,
				int                      estado,
				Ubicacion                    ubicacion,
				ArrayList<Alerta> alertas)
	{
		this.id          = id;
		this.nombre      = nombre;
		this.descripcion = descripcion;
		this.fecha       = fecha;
		this.fechaAlerta = fechaAlerta;
		this.kmsTarde    = kmsTarde;
		this.estado      = estado;
		this.ubicacion       = ubicacion;
		this.alertas    = alertas;
	}
	
	public Evento(int 					 id,
			String              	 nombre,
			String                   descripcion,
			long                     msFecha,
			long                     msFechaAlerta,
			float                    kmsTarde,
			int                      estado,
			Ubicacion                    ubicacion,
			ArrayList<Alerta> alertas)
	{
		this(id,
			 nombre,
			 descripcion,
			 new Date(msFecha),
			 new Date(msFechaAlerta),
			 kmsTarde,
			 estado,
			 ubicacion,
			 alertas);
	}
	
	public Evento(int    id,
			    String nombre,
			    String descripcion,
			    Date   fecha,
			    Date   fechaAlerta,
			    float  kmsTarde,
			    int    estado)
	{
		this(id,
			 nombre,
			 descripcion,
			 fecha,
			 fechaAlerta,
			 kmsTarde,
			 estado,
			 null,
			 new ArrayList<Alerta>());
	}
	
	public Evento(int id,
		    String nombre,
		    String descripcion,
		    long msFecha,
		    long msFechaAlerta,
		    float kmsTarde,
		    int estado)
	{
		this(id,
			 nombre,
			 descripcion,
			 new Date(msFecha),
			 new Date(msFecha),
			 kmsTarde,
			 estado);
	}

	
	// Getters & Setters
	public int getId() 
	{
		return id;
	}
	
	public void setId(int id)
	{
		this.id = id;
	}
	
	public String getNombre() 
	{
		return nombre;
	}
	
	public void setNombre(String nombre)
	{
		this.nombre = nombre;
	}
	
	public String getDescripcion()
	{
		return descripcion;
	}
	
	public void setDescripcion(String descripcion) 
	{
		this.descripcion = descripcion;
	}
	
	public Date getFecha()
	{
		return fecha;
	}
	
	public void setFecha(Date fecha) 
	{
		this.fecha = fecha;
	}
	
	public Date getFechaAlerta() 
	{
		return fechaAlerta;
	}
	
	public void setFechaAlerta(Date fechaAlerta) 
	{
		this.fechaAlerta = fechaAlerta;
	}
	
	public float getKmsTarde()
	{
		return this.kmsTarde;
	}
	
	public void setKmsTarde(float kmsTarde)
	{
		this.kmsTarde = kmsTarde;
	}
	
	public Ubicacion getUbicacion()
	{
		return ubicacion;
	}
	
	public void setUbicacion(Ubicacion ubicacion) 
	{
		this.ubicacion = ubicacion;
	}
	
	public int getEstado()
	{
		return this.estado;
	}
	
	public void setEstado(int estado)
	{
		this.estado = estado;
	}
	
	public boolean isTratada()
	{
		return this.estado == ESTADO_FINALIZADO;
	}
	
	public ArrayList<Alerta> getAcciones()
	{
		return this.alertas;
	}
	
	public void addAccion(Alerta a)
	{
		this.alertas.add(a);
	}
	
	public void removeAccion(Alerta a)
	{
		this.alertas.remove(a);
	}

	@Override
	public int compareTo(Evento c) 
	{
		if(this.getEstado() != c.getEstado())
			return this.getEstado() > c.getEstado() ? -1 : 1;
			
		// Ante igual prioridad del estado, la prioridad la marca la fecha
		return this.fecha.compareTo(c.fecha); 
	}
	
	// Public methods
	public boolean llegoTarde(Date d, GeoPoint currentLocation)
	{
		if(isTratada())
			return false;
		
		Date now = new Date();
		
		if(now.before(this.fechaAlerta)) // si todavia no hemos llegado a la fecha de alerta, no llegamos tarde
			return false;
		
		// ya estamos dentro de la fecha de alerta
		if(Utilidades.getDistanciaKm(this.ubicacion.getLocation(), currentLocation) > this.kmsTarde)
			return true;
		return false;
	}
	
	
	public boolean llegoTarde(GeoPoint geop) {
	      boolean es = false;
	      Date fecha = new Date();
	      if(!fecha.before(this.fechaAlerta) && !fecha.after(this.fecha) && Utilidades.getDistanciaKm(this.ubicacion.getLocation(), geop) > (double)this.kmsTarde) {
	         es = true;
	      }

	      return es;
	   }
	
	public boolean isFechasOk()
	{
		if(this.fecha == null || this.fechaAlerta == null)
			return false;
		
		// la fecha de la Evento ha de ser posterior a la de la alerta
		return this.fecha.after(this.fechaAlerta);
	}

	
	// IListViewBasicInfo Methods
	
	@Override
	public int getElementoImagenID() {	
		switch(this.getEstado())
		{
			case ESTADO_IMPUNTUAL: 
				return R.drawable.alerta;
				
			case ESTADO_PUNTUAL:
				return R.drawable.calendarojo_;
				
			case ESTADO_FINALIZADO:      
				return R.drawable.eventermina;
				
			default:                    
				return R.drawable.calendarverde; 
		}
		
	}

	@Override
	public String getElementoPrincipal() 
	{
		return this.getNombre();
	}

	@Override
	public String getElementoSecundario() 
	{
		String f;
		if(this.fecha == null)
			f = "[null]";
		else
		{		
			calendar.setTimeInMillis(this.fecha.getTime());
			int yy = calendar.get(Calendar.YEAR);
			int MM = calendar.get(Calendar.MONTH);
			int dd = calendar.get(Calendar.DAY_OF_MONTH);
			int hh = calendar.get(Calendar.HOUR_OF_DAY);
			int mm = calendar.get(Calendar.MINUTE);
			f = String.format("%02d/%02d/%d - %02d:%02d", dd, MM+1, yy, hh, mm);
		}
		
		//String f = this.fecha != null ? DateFormat.format("dd/MM/yyyy - HH:mm", this.fecha).toString() : "[null]";
		String estado = getStrEstado();
		return f + "\n" + estado;		
	}
	
	@Override
	public String toString()
	{
		return this.getElementoSecundario() + "\nAlerta: " + DateFormat.format("dd/MM/yyyy hh:mm", this.fechaAlerta);
	}
	
	// Util Methods
	public String getStrFecha()
	{		
		return this.fecha != null ? DateFormat.format("dd/MM/yyyy", this.fecha).toString() : "[null]";
	}
	
	public String getStrHour()
	{
		if(this.fecha == null)
			return "[null]";
		
		calendar.setTimeInMillis(this.fecha.getTime());
		int hh = calendar.get(Calendar.HOUR_OF_DAY);
		int mm = calendar.get(Calendar.MINUTE);
		return String.format("%02d:%02d", hh, mm);
	}
	
	public String getStrFechaWarning()
	{
		return this.fechaAlerta != null ? DateFormat.format("dd/MM/yyyy", this.fechaAlerta).toString() : "[null]";
	}
	
	public String getStrHourWarning()
	{
		if(this.fechaAlerta == null)
			return "[null]";
		
		calendar.setTimeInMillis(this.fechaAlerta.getTime());
		int hh = calendar.get(Calendar.HOUR_OF_DAY);
		int mm = calendar.get(Calendar.MINUTE);
		return String.format("%02d:%02d", hh, mm);

	}
	
	public void resetToCorrectState()
	{
		Date now = new Date();
		
		if(now.before(fechaAlerta))
			this.estado = ESTADO_PENDIENTE;
		else if(now.before(fecha))
			this.estado = ESTADO_IMPUNTUAL;
		else
			this.estado = ESTADO_FINALIZADO;
	}
	
	public boolean tratar()
	{
		Date now = new Date();
		
		if(now.after(fechaAlerta) && now.before(fecha) && this.estado == ESTADO_PENDIENTE)
			return true;
		
		return false;
	}
	
	public boolean tratarFin() // nos dice si ya ha pasado la fecha y hemos de modificarle su estado a tratada
	{
		Date now = new Date();
		
		if(now.after(fecha) && this.estado != ESTADO_FINALIZADO)
			return true;
		return false;
	}
	
	
	
	
	public String getStrEstado() {
	      String var1;
	      switch(this.getEstado()) {
	      case 1:
	         var1 = STR_ESTADO_TRATADA;
	         break;
	      case 2:
	      default:
	         var1 = STR_ESTADO_NO_TRATADA;
	         break;
	      case 3:
	         var1 = STR_ESTADO_LLEGAS_EN_HORA;
	         break;
	      case 4:
	         var1 = STR_ESTADO_LLEGAS_TARDE;
	      }

	      return var1;
	   }

	
	
	
}
