package es.rediris.forja.jleon.arrivalert;
 
import java.util.ArrayList;
import java.util.Iterator;

import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Paint;
import android.graphics.Paint.Style;
import android.graphics.Point;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.ItemizedOverlay;
import com.google.android.maps.MapView;
import com.google.android.maps.OverlayItem;

 

	public class ItemizedOverlayArrivAlert extends ItemizedOverlay<OverlayItem> {
	
	private ArrayList<OverlayItem> overlayItemList = new ArrayList<OverlayItem>();
	private MapView mv;
	private Bitmap pica;    
	private Paint innerPaint, borderPaint, textPaint;
	private Bitmap marca;
	private GeoPoint gpoint;
    private OverlayItem overItem;
	 
	
	public ItemizedOverlayArrivAlert(MapView mv, Drawable marker, Drawable currentLocationMarker) {
		super(boundCenterBottom(marker));
		this.mv  = mv;
		this.pica = Utilidades.dibujaBitmap(marker);
		this.marca = Utilidades.dibujaBitmap(currentLocationMarker);
		populate();	
	}
	
	
	
	public GeoPoint getCurrentLocation() {
		return this.gpoint;
	}
	
	public void setCurrentLocation(GeoPoint currentLocation) {
		
		this.gpoint = currentLocation;
		String title = this.mv.getContext().getString(R.string.str_current_location);
		OverlayItem item = new OverlayItem(this.gpoint, title, "");
		
		if(this.overlayItemList.size() > 0)
			this.overlayItemList.set(0, item);
		else
			this.overlayItemList.add(item);
		
		populate();
	}
	 
	
	public void addItem(GeoPoint p, String title, String snippet) {
		
		OverlayItem newItem = new OverlayItem(p, title, snippet);
		this.overlayItemList.add(newItem);
		populate();
	}
	
	
	public void clearExceptCurrentLocation() {
		
		while(this.overlayItemList.size() > 1)
			this.overlayItemList.remove(1);
	}
	
	
	@Override
	protected OverlayItem createItem(int i) {
		return overlayItemList.get(i);
	}
	 
	@Override
	public int size() {	
		return overlayItemList.size();
	}
	 
	@Override
	public void draw(Canvas canvas, MapView mapView, boolean shadow) {
	
		drawMapLocations(canvas, mapView, shadow);
   		drawInfoWindow(canvas, mapView, shadow);
	}
	
	
	@Override
    public boolean onTap(GeoPoint p, MapView mapView) {		
			
			boolean isRemovePriorPopup = this.overItem != null;  

			this.overItem = getHitMapLocation(mapView, p);
			if (isRemovePriorPopup || this.overItem != null) {
				
				mapView.invalidate();
			}	
			
			return this.overItem != null;
    }
	
	

    private OverlayItem getHitMapLocation(MapView mapView, GeoPoint	tapPoint) {    	
    	
    	OverlayItem hitMapLocation = null;
		
    	RectF hitTestRecr = new RectF();
		Point screenCoords = new Point();
    	Iterator<OverlayItem> iterator = this.overlayItemList.iterator();
    	
    	Bitmap bm = this.marca;
    	
    	
    	while(iterator.hasNext()) {
    		OverlayItem testLocation = iterator.next();    		
    		
    		// This is used to translate the map's lat/long coordinates to screen's coordinates
    		mapView.getProjection().toPixels(testLocation.getPoint(), screenCoords);

	    	// Create a testing Rectangle with the size and coordinates of our icon
	    	// Set the testing Rectangle with the size and coordinates of our on screen icon
    		hitTestRecr.set(-bm.getWidth()/2,-bm.getHeight(),bm.getWidth()/2,0);
    		hitTestRecr.offset(screenCoords.x,screenCoords.y);

	    	//  At last test for a match between our Rectangle and the location clicked by the user
    		mapView.getProjection().toPixels(tapPoint, screenCoords);
    		if (hitTestRecr.contains(screenCoords.x,screenCoords.y)) {
    			hitMapLocation = testLocation;
    			break;
    		}
    		
    		bm = this.pica; // los siguientes serán las marcas que hemos puesto
    	}
    	
    	//  Finally clear the new MouseSelection as its process finished
    	tapPoint = null;
    	
    	return hitMapLocation; 
    }
    
    private void drawMapLocations(Canvas canvas, MapView mapView, boolean shadow) {    	
		Iterator<OverlayItem> iterator = this.overlayItemList.iterator();
		Point screenCoords = new Point();
		
		// El primer bitmap siempre corresponderá a la posición actual
		Bitmap bm = this.marca;
		
    	while(iterator.hasNext()) {	   
    		OverlayItem location = iterator.next();
    		mapView.getProjection().toPixels(location.getPoint(), screenCoords);
			
	    	if (shadow) {
	    		// no pintamos ninguna sombra
	    		// Offset the shadow in the y axis as it is angled, so the base is at x=0
	    		//canvas.drawBitmap(mShadowIcon, screenCoords.x, screenCoords.y - mShadowIcon.getHeight(),null);
	    	} 
	    	else {
    			canvas.drawBitmap(bm, 
    							  screenCoords.x - bm.getWidth()/2,
    							  screenCoords.y - bm.getHeight(),
    							  null);
	    	}
	    	
	    	bm = this.pica; // los siguientes corresponderan a la marca de favoritos
    	}
    }

    private void drawInfoWindow(Canvas canvas, MapView	mapView, boolean shadow) {    	
    	if (overItem != null) {
    		if (shadow) {
    			//  Skip painting a shadow
    		}
    		else {
				//  First we need to determine the screen coordinates of the selected MapLocation
				Point selDestinationOffset = new Point();
				mapView.getProjection().toPixels(overItem.getPoint(), selDestinationOffset);
		    	
		    	//  Setup the info window
				int INFO_WINDOW_WIDTH  = 175;
				int INFO_WINDOW_HEIGHT = 40;
				int TEXT_OFFSET_X      = 10;
				int TEXT_OFFSET_Y      = 15;
				
				Rect r   = new Rect();
				
				String title = overItem.getTitle();
				// String desc  = overItem.getSnippet();
				// String s = overItem.getTitle().toUpperCase() + " - " + overItem.getSnippet();
				String s = title;
				Paint p  = getmTextPaint();
				p.getTextBounds(s, 0, s.length()-1, r);
				
				INFO_WINDOW_HEIGHT = r.height()+ TEXT_OFFSET_Y*2;
				INFO_WINDOW_WIDTH  = r.width() + TEXT_OFFSET_X*2;				
				
				RectF infoWindowRect  = new RectF(0, 0, INFO_WINDOW_WIDTH, INFO_WINDOW_HEIGHT);				
				int infoWindowOffsetX = selDestinationOffset.x-INFO_WINDOW_WIDTH/2;
				int infoWindowOffsetY = selDestinationOffset.y-INFO_WINDOW_HEIGHT-pica.getHeight();
				infoWindowRect.offset(infoWindowOffsetX,infoWindowOffsetY);
					
								
				//  Drawing the inner info window
				canvas.drawRoundRect(infoWindowRect, 5, 5, getmInnerPaint());
				
				//  Drawing the border for info window
				canvas.drawRoundRect(infoWindowRect, 5, 5, getmBorderPaint());
				
				//  Draw the MapLocation's name
				canvas.drawText(s,
						        infoWindowOffsetX+TEXT_OFFSET_X,
						        infoWindowOffsetY+TEXT_OFFSET_Y,
						        p);
			}
    	}
    }

	private Paint getmInnerPaint() {
		if (innerPaint == null) {
			innerPaint = new Paint();
			innerPaint.setARGB(225, 50, 50, 50); //inner color
			innerPaint.setAntiAlias(true);
		}
		return innerPaint;
	}

	private Paint getmBorderPaint() {
		if (borderPaint == null) {
			borderPaint = new Paint();
			borderPaint.setARGB(255, 255, 255, 255);
			borderPaint.setAntiAlias(true);
			borderPaint.setStyle(Style.STROKE);
			borderPaint.setStrokeWidth(2);
		}
		return borderPaint;
	}

	private Paint getmTextPaint() {
		if (textPaint == null)
		{
			textPaint = new Paint();
			textPaint.setARGB(255, 255, 255, 255);
			textPaint.setAntiAlias(true);
			textPaint.setFakeBoldText(true);
		}
		return textPaint;
	} 
}
