package es.rediris.forja.jleon.arrivalert;

import java.util.ArrayList;

import android.content.Context;
import android.graphics.drawable.Drawable;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;


 public class Localizador implements LocationListener {
	
    private MapController mapController;
    private MapView mapView;
	private LocationManager manejador;	
	private GeoPoint lastLocation;
	private Context context;
	private ItemizedOverlayArrivAlert itemizedOverlay;

	
	
    public Localizador(MapView mapView, LocationManager manejador, Context context) {
    	
    	this.mapView = mapView;
    	this.mapController = mapView.getController();
    	this.manejador = manejador;
    	this.context = context;
    	this.lastLocation = null;

    	// controles de vision
    	this.mapView.setBuiltInZoomControls(true);
    	this.mapView.setSatellite(false);            
    	this.mapView.setTraffic(false);         	
    	this.mapController.setZoom(17);   
    	
    	this.initMarkers();
    }
    
    /**
     * Para inicializar los objetos que nos permitirán añadir marcas
     */
    private void initMarkers() {
    	Drawable marker = this.context.getResources().getDrawable(R.drawable.marker_red); //localizado en
    	Drawable currLocMarker = this.context.getResources().getDrawable(R.drawable.marcalugar); //cruz roja
    	Utilidades.setIntrinsicSize(marker);
    	Utilidades.setIntrinsicSize(currLocMarker);
    
        this.itemizedOverlay = new ItemizedOverlayArrivAlert(this.getMapView(), marker, currLocMarker);
        this.mapView.getOverlays().add(itemizedOverlay);
    }
	
	
	
    
    public MapController getMapController() {
		return mapController;
	}


	public MapView getMapView() {
		return mapView;
	}


	public LocationManager getManejador() {
		return manejador;
	}
	
	public GeoPoint getLastLocation() {
		return this.lastLocation;
	}
	
	
	public void addUbicacion(Ubicacion l) {
		this.itemizedOverlay.addItem(l.getLocation(), l.getNombre(), l.getDescripcion());
	}
	
	public void addUbicacion(ArrayList<Ubicacion> lugares) {
		//no borrar nuestra posicion + agregar ubicaciones + dibuja mapa
		this.itemizedOverlay.clearExceptCurrentLocation();
		for(Ubicacion l:lugares)
			addUbicacion(l);
		this.mapView.invalidate();
	}


	
	// Metodos de LocationListener       

    @Override
    public void onLocationChanged(Location location) {
		   int lat = (int) (location.getLatitude()  * 1E6);
		   int lng = (int) (location.getLongitude() * 1E6);
		   this.lastLocation = new GeoPoint(lat, lng);
		   
		   // Refrescar posicion actual
	       if(this.lastLocation != null) {
	    	   if(this.itemizedOverlay.getCurrentLocation() == null) {
	    			this.mapController.setCenter(this.lastLocation);	
	    	   }
	    	   
	    	   
	    	   	this.itemizedOverlay.setCurrentLocation(this.lastLocation);
	            this.mapView.invalidate();
	       }
    }       

    @Override
    public void onProviderDisabled(String provider) {}       

    @Override
    public void onProviderEnabled(String provider) {}       

    @Override
    public void onStatusChanged(String provider, int status, Bundle extras){}
    
    
    
    
    public void activar() {
    	// TIENE QUE FUNCIONAR CON GPS Y POSICIONAMIENTO BASADO EN REDES
    	this.manejador.requestLocationUpdates(LocationManager.GPS_PROVIDER, 10000, 1, this);   
    	this.manejador.requestLocationUpdates(LocationManager.NETWORK_PROVIDER, 10000, 1, this);
    }
    
    public void desactivar() {
    	this.manejador.removeUpdates(this);
    }

}
