package es.rediris.forja.jleon.arrivalert;

import java.util.ArrayList;

import android.app.ListActivity;
import android.content.Intent;
import android.net.Uri;
import android.os.Bundle;
import android.view.ContextMenu;
import android.view.ContextMenu.ContextMenuInfo;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.Window;
import android.widget.AdapterView;
import android.widget.ListView;
import android.widget.TextView;
import android.widget.Toast;
import com.google.android.maps.GeoPoint;

public class UbicacionesActivity extends ListActivity 
{
	// Attributes
	private AdaptarElementos mAdaptador;
	private int mode;

	@Override
	protected void onCreate(Bundle savedInstanceState) 
	{
		super.onCreate(savedInstanceState);
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		this.setContentView(R.layout.ubicaciones);
		
		refreshAdaptador();
		
		// Registramos el context menu
	    ListView list = (ListView)this.findViewById(android.R.id.list);
	    registerForContextMenu(list);
	    
		// Obtenemos el modo de edición
		Bundle extras = this.getIntent().getExtras();
		int m = extras == null ? -1 : extras.getInt(Constantes.CTE_MODO);
		if(m == Constantes.CTE_SELECCION)
			this.mode = Constantes.CTE_SELECCION;
		else
			this.mode = Constantes.CTE_VISTA;
		
		// Solo cambiamos las cosas si estamos en mode select
		// En mode view los controles ya tienen los valores correctos
		if(this.mode == Constantes.CTE_SELECCION)
		{
				// Ponemos los valores en los TextView
				((TextView)findViewById(R.id.lblHeader)).setText(getString(R.string.str_select_place));
		}
	}
	
	private void refreshAdaptador()
	{
		// Base de datos actual
		BDUbicacion bbdd = new BDUbicacion(this);
        
        // Adaptador        
        ArrayList<IListaElementos> list = new ArrayList<IListaElementos>(bbdd.getAllUbicacion()); // tenemos que convertir el tipo de lista
        this.mAdaptador = new AdaptarElementos(this, list);
        
        // Set the adapter
        setListAdapter(this.mAdaptador);
        
        bbdd.close();
	}
	
	
	public void onClickOnAddPlace(View v)
	{
        Intent i = new Intent(this, ConfiguraUbicacionActivity.class);
        this.startActivityForResult(i, Constantes.CTE_REG_UBICACION);
	}
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data)
	{
		if(resultCode == RESULT_OK && requestCode == Constantes.CTE_REG_UBICACION)
		{
			refreshAdaptador(); // si ha anyadido bien el nuevo sitio, refrescamos desde la base de datos
		}
	}
	
	public void onClickOnViewAllPlaces(View v)
	{
        Intent i = new Intent(this, MapaActivity.class);
        i.putExtra(Constantes.CTE_MODO, MapaActivity.MODE_ONLY_VIEW); // lo abrimos en modo sólo vista de los puntos guardados        
        this.startActivity(i);
	}
	
	
	@Override 
	protected void onListItemClick(ListView listView, View view, int position, long id) 
	{
		super.onListItemClick(listView, view, position, id);
		
		Object o = getListAdapter().getItem(position); 							// Cogemos el item en la posición indicada		
		//String s = String.format("Selección: %d - %s", position, o.toString()); // Creamos un string con info
		//Toast.makeText(this, s, Toast.LENGTH_LONG).show();						// Mostramos la Toast
		
		// Devolvemos el id y el nombre del lugar seleccionado		
		Ubicacion l = (Ubicacion)o;
		if(mode == Constantes.CTE_SELECCION)
			seleccionarUbicacion(l);
		else
			editUbicacion(l);
	}
	
	
	private void seleccionarUbicacion(Ubicacion l)
	{
		Intent i = new Intent();
		i.putExtra(Constantes.CTE_UBICACION_NOMBRE, l.getNombre());
		i.putExtra(Constantes.CTE_UBICACION_ID,     l.getId());
		this.setResult(RESULT_OK, i);
		this.finish();
	}
	
	private void editUbicacion(Ubicacion l)
	{
		  Intent i = new Intent(this, ConfiguraUbicacionActivity.class);
		  i.putExtra("id_lugar", l.getId());  // editamos el lugar con el id especificado
	      this.startActivityForResult(i, Constantes.CTE_REG_UBICACION);
	}
	
	private void deleteUbicacion(Ubicacion l)
	{
		// Borramos la fila de la base de datos
		BDUbicacion bbdd = new BDUbicacion(this);
        bbdd.deleteUbicacion(l);	        
        bbdd.close();
        
        // refrescamos nuestra lista
        refreshAdaptador();
	}
	
	private void viewUbicacion(Ubicacion l)
	{
				  GeoPoint p = l.getLocation();		  
		  if(p != null)
		  {
			  Uri uri = Uri.parse(l.getUriString());
		  	  Intent intent = new Intent(Intent.ACTION_VIEW, uri);		  	  
		  	  startActivity(intent);
		  }
		  else
		  {
			  Toast.makeText(this, "No location", Toast.LENGTH_SHORT).show();
		  }
	}
	
	
	
	// Context Menu
	
	@Override
	public void onCreateContextMenu(ContextMenu menu, View v, ContextMenuInfo menuInfo) 
	{
		if (v.getId()==android.R.id.list) 
		{
			AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)menuInfo;
			Ubicacion l = (Ubicacion)this.mAdaptador.getItem(info.position);
			menu.setHeaderTitle(l.getNombre());
			
			// Add the options
			menu.add(Menu.NONE, 0, 0, R.string.str_select);
			menu.add(Menu.NONE, 1, 1, R.string.str_delete);
			menu.add(Menu.NONE, 2, 2, R.string.str_edit);
			if(l.getLocation() != null)
				menu.add(Menu.NONE, 3, 3, R.string.str_show_in_map);
		}
	}
	
	
	@Override
	public boolean onContextItemSelected(MenuItem item)
	{
	  AdapterView.AdapterContextMenuInfo info = (AdapterView.AdapterContextMenuInfo)item.getMenuInfo();
	  int menuItemId = item.getItemId();
	  Ubicacion l = (Ubicacion)this.mAdaptador.getItem(info.position);
	  
	  switch(menuItemId)
	  {
		  case 0: seleccionarUbicacion(l); break;
		  case 1: deleteUbicacion(l);      break;
		  case 2: editUbicacion(l);        break;
		  case 3: viewUbicacion(l);        break;
	  }
  
	  return true;
	}
}
